@@ -16,7 +16,7 @@ pub fn consume_header_until<R: Read>(
     reader: R,
     pattern: &Regex,
 ) -> io::Result<Option<(u64, RecombobulatedReader<R>)>> {
-    let mut line_reader = simd_csv::LineReader::new(reader);
+    let mut line_reader = simd_csv::LineReader::from_reader(reader);
     let mut pos = line_reader.position();
     let mut header_opt: Option<Vec<u8>> = None;
 
@@ -48,7 +48,7 @@ pub fn consume_header_while<R: Read>(
     reader: R,
     pattern: &Regex,
 ) -> io::Result<Option<(u64, RecombobulatedReader<R>)>> {
-    let mut line_reader = simd_csv::LineReader::new(reader);
+    let mut line_reader = simd_csv::LineReader::from_reader(reader);
     let mut pos = line_reader.position();
     let mut header_opt: Option<Vec<u8>> = None;
 
@@ -80,7 +80,7 @@ pub fn consume_lines<R: Read>(
     reader: R,
     limit: usize,
 ) -> io::Result<Option<(u64, RecombobulatedReader<R>)>> {
-    let mut line_reader = simd_csv::LineReader::new(reader);
+    let mut line_reader = simd_csv::LineReader::from_reader(reader);
     let mut pos = line_reader.position();
     let mut seen: usize = 0;
     let mut header_opt: Option<Vec<u8>> = None;
